/**********************************
 *
 * Copyright (C) 2004 Paul Pierce
 *
 * Permission instanceof hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software instanceof furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 *********************************/

package com.piercefuller.a7trk.a7tview;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Label;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import com.piercefuller.a7trk.decode.DataFile;
import com.piercefuller.a7trk.decode.MultiSource;
import com.piercefuller.a7trk.decode.Source;
import com.piercefuller.a7trk.decode.TapeParameters;
import com.piercefuller.a7trk.decode.TapeParametersList;

/// <summary>
/// Summary description for ConfigDialog.
/// </summary>
public class ConfigDialog extends JDialog {
    private A7tView        parent;
    private int            nchannel      = 0;
    private TapeParameters tp            = null;
    private Label[]        channelLabels = null;
    private JTextField[]   skewTextBoxes = null;
    private JTextField[]   clipTextBoxes = null;
    private JTextField[]   baseTextBoxes = null;

    private JLabel         rawDataSectionLabel;
    private JLabel         rawDataScaleLabel;
    private JTextField     rawDataScaleTextbox;
    private JTextField     lowSpaceTextbox;
    private JLabel         lowSpaceLabel;
    private JLabel         parametersSectionLabel;
    private JButton        updateButton;
    private JTextField     processedScaleTextbox;
    private JLabel         processedScaleLabel;
    private JLabel         processedSectionLabel;
    private JTextField     gainTextbox;
    private JLabel         gainLabel;
    private JTextField     widthTextbox;
    private JLabel         widthLabel;
    private JTextField     gatherTextbox;
    private JLabel         gatherLabel;
    private JLabel         crowdLabel;
    private JTextField     crowdTextbox;
    private JLabel         highSpaceLabel;
    private JTextField     highSpaceTextbox;
    private JButton        insertButton;
    private JButton        setEndButton;
    private JButton        deleteButton;
    private JLabel         maxSpaceLabel;
    private JLabel         minGapLabel;
    private JLabel         maxJunkLabel;
    private JTextField     maxSpaceTextbox;
    private JTextField     minGapTextbox;
    private JTextField     maxJunkTextbox;
    private JLabel         startPositionLabel;
    private JLabel         endPositionLabel;
    private JLabel         setEndPositionLabel;
    private JLabel         insertPositionLabel;
    private JCheckBox      compensateCheckbox;

    public ConfigDialog(final A7tView p) {
        parent = p;

        // Font f = Font.getFont("SansSerif").deriveFont(Font.BOLD).deriveFont(8.25F);
        Font f = getFont();

        rawDataSectionLabel = new JLabel();
        rawDataSectionLabel.setFont(f);
        rawDataSectionLabel.setName("rawDataSectionLabel");
        rawDataSectionLabel.setText("Raw Data");

        rawDataScaleLabel = new JLabel();
        rawDataScaleLabel.setName("rawDataScaleLabel");
        rawDataScaleLabel.setText("Scale");

        rawDataScaleTextbox = new JTextField();
        rawDataScaleTextbox.setName("rawDataScaleTextbox");
        rawDataScaleTextbox.setColumns(5);
        rawDataScaleTextbox.setHorizontalAlignment(SwingConstants.RIGHT);

        processedSectionLabel = new JLabel();
        processedSectionLabel.setFont(f);
        processedSectionLabel.setName("processedSectionLabel");
        processedSectionLabel.setText("Lump / Baseline");

        processedScaleLabel = new JLabel();
        processedScaleLabel.setName("processedScaleLabel");
        processedScaleLabel.setText("Scale");

        processedScaleTextbox = new JTextField();
        processedScaleTextbox.setName("processedScaleTextbox");
        processedScaleTextbox.setColumns(5);
        processedScaleTextbox.setHorizontalAlignment(SwingConstants.RIGHT);

        parametersSectionLabel = new JLabel();
        parametersSectionLabel.setFont(f);
        parametersSectionLabel.setName("parametersSectionLabel");
        parametersSectionLabel.setText("Parameters");

        gainLabel = new JLabel();
        gainLabel.setName("gainLabel");
        gainLabel.setText("Gain");

        gainTextbox = new JTextField();
        gainTextbox.setName("gainTextbox");
        gainTextbox.setColumns(5);
        gainTextbox.setHorizontalAlignment(SwingConstants.RIGHT);

        widthLabel = new JLabel();
        widthLabel.setName("widthLabel");
        widthLabel.setText("Width");

        widthTextbox = new JTextField();
        widthTextbox.setName("widthTextbox");
        widthTextbox.setColumns(5);
        widthTextbox.setHorizontalAlignment(SwingConstants.RIGHT);

        crowdLabel = new JLabel();
        crowdLabel.setName("crowdLabel");
        crowdLabel.setText("Crowd");

        crowdTextbox = new JTextField();
        crowdTextbox.setName("crowdTextbox");
        crowdTextbox.setColumns(5);
        crowdTextbox.setHorizontalAlignment(SwingConstants.RIGHT);

        gatherLabel = new JLabel();
        gatherLabel.setName("gatherLabel");
        gatherLabel.setText("Gather");

        gatherTextbox = new JTextField();
        gatherTextbox.setName("gatherTextbox");
        gatherTextbox.setColumns(5);
        gatherTextbox.setHorizontalAlignment(SwingConstants.RIGHT);

        lowSpaceLabel = new JLabel();
        lowSpaceLabel.setName("lowSpaceLabel");
        lowSpaceLabel.setText("Low Sp");

        lowSpaceTextbox = new JTextField();
        lowSpaceTextbox.setName("lowSpaceTextbox");
        lowSpaceTextbox.setColumns(5);
        lowSpaceTextbox.setHorizontalAlignment(SwingConstants.RIGHT);

        highSpaceLabel = new JLabel();
        highSpaceLabel.setName("highSpaceLabel");
        highSpaceLabel.setText("High Sp");

        highSpaceTextbox = new JTextField();
        highSpaceTextbox.setName("highSpaceTextbox");
        highSpaceTextbox.setColumns(5);
        highSpaceTextbox.setHorizontalAlignment(SwingConstants.RIGHT);

        maxSpaceLabel = new JLabel();
        maxSpaceLabel.setName("maxSpaceLabel");
        maxSpaceLabel.setText("MaxSpace");

        maxSpaceTextbox = new JTextField();
        maxSpaceTextbox.setName("maxSpaceTextbox");
        maxSpaceTextbox.setColumns(5);
        maxSpaceTextbox.setHorizontalAlignment(SwingConstants.RIGHT);

        minGapLabel = new JLabel();
        minGapLabel.setName("minGapLabel");
        minGapLabel.setText("MinGap");

        minGapTextbox = new JTextField();
        minGapTextbox.setName("minGapTextbox");
        minGapTextbox.setColumns(5);
        minGapTextbox.setHorizontalAlignment(SwingConstants.RIGHT);

        maxJunkLabel = new JLabel();
        maxJunkLabel.setName("maxJunkLabel");
        maxJunkLabel.setText("MaxJunk");

        maxJunkTextbox = new JTextField();
        maxJunkTextbox.setName("maxJunkTextbox");
        maxJunkTextbox.setColumns(5);
        maxJunkTextbox.setHorizontalAlignment(SwingConstants.RIGHT);

        insertButton = new JButton();
        insertButton.setName("insertButton");
        insertButton.setText("Start New");
        insertButton.addActionListener(button2_Click);

        setEndButton = new JButton();
        setEndButton.setName("setEndButton");
        setEndButton.setText("Set End");
        setEndButton.addActionListener(button3_Click);

        updateButton = new JButton();
        updateButton.setName("updateButton");
        updateButton.setText("Update");
        updateButton.addActionListener(button1_Click);

        deleteButton = new JButton();
        deleteButton.setName("deleteButton");
        deleteButton.setText("Delete");
        deleteButton.addActionListener(button4_Click);

        startPositionLabel = new JLabel();
        startPositionLabel.setName("startPositionLabel");

        endPositionLabel = new JLabel();
        endPositionLabel.setName("endPositionLabel");

        setEndPositionLabel = new JLabel();
        setEndPositionLabel.setName("setEndPositionLabel");

        insertPositionLabel = new JLabel();
        insertPositionLabel.setName("insertPositionLabel");

        compensateCheckbox = new JCheckBox();
        compensateCheckbox.setName("compensateCheckbox");
        compensateCheckbox.setText("Compensate");

        //
        // ConfigDialog
        //

        Insets insets = new Insets(0, 3, 0, 6);

        JPanel parmBox = new JPanel(new GridBagLayout());
        GridBagConstraints parmSectionConstraints = new GridBagConstraints();
        parmSectionConstraints.gridwidth = 2;
        parmSectionConstraints.gridx = 0;
        parmSectionConstraints.fill = GridBagConstraints.HORIZONTAL;
        parmSectionConstraints.anchor = GridBagConstraints.LINE_START;
        parmSectionConstraints.insets = insets;
        GridBagConstraints parmLabelConstraints = new GridBagConstraints();
        parmLabelConstraints.anchor = GridBagConstraints.LINE_START;
        parmLabelConstraints.gridx = 0;
        parmLabelConstraints.insets = parmSectionConstraints.insets;
        GridBagConstraints parmItemConstraints = new GridBagConstraints();
        parmItemConstraints.anchor = GridBagConstraints.LINE_START;
        parmItemConstraints.gridx = 1;
        parmItemConstraints.fill = GridBagConstraints.HORIZONTAL;
        parmItemConstraints.insets = parmSectionConstraints.insets;

        parmBox.add(rawDataSectionLabel, parmSectionConstraints);
        parmBox.add(rawDataScaleLabel, parmLabelConstraints);
        parmBox.add(rawDataScaleTextbox, parmItemConstraints);

        parmBox.add(processedSectionLabel, parmSectionConstraints);
        parmBox.add(processedScaleLabel, parmLabelConstraints);
        parmBox.add(processedScaleTextbox, parmItemConstraints);

        parmBox.add(parametersSectionLabel, parmSectionConstraints);
        parmBox.add(gainLabel, parmLabelConstraints);
        parmBox.add(gainTextbox, parmItemConstraints);
        parmBox.add(widthLabel, parmLabelConstraints);
        parmBox.add(widthTextbox, parmItemConstraints);
        parmBox.add(crowdLabel, parmLabelConstraints);
        parmBox.add(crowdTextbox, parmItemConstraints);
        parmBox.add(gatherLabel, parmLabelConstraints);
        parmBox.add(gatherTextbox, parmItemConstraints);
        parmBox.add(lowSpaceLabel, parmLabelConstraints);
        parmBox.add(lowSpaceTextbox, parmItemConstraints);
        parmBox.add(highSpaceLabel, parmLabelConstraints);
        parmBox.add(highSpaceTextbox, parmItemConstraints);
        parmBox.add(maxSpaceLabel, parmLabelConstraints);
        parmBox.add(maxSpaceTextbox, parmItemConstraints);
        parmBox.add(minGapLabel, parmLabelConstraints);
        parmBox.add(minGapTextbox, parmItemConstraints);
        parmBox.add(maxJunkLabel, parmLabelConstraints);
        parmBox.add(maxJunkTextbox, parmItemConstraints);

        JPanel buttonBox = new JPanel(new GridBagLayout());
        GridBagConstraints buttonTop = new GridBagConstraints();
        buttonTop.gridy = 1;
        GridBagConstraints buttonMiddle = new GridBagConstraints();
        buttonMiddle.gridy = 2;
        GridBagConstraints buttonBottom = new GridBagConstraints();
        buttonBottom.gridy = 0;

        buttonBox.add(insertButton, buttonTop);
        buttonBox.add(setEndButton, buttonTop);
        buttonBox.add(updateButton, buttonTop);
        buttonBox.add(deleteButton, buttonTop);
        buttonBox.add(insertPositionLabel, buttonMiddle);
        buttonBox.add(setEndPositionLabel, buttonMiddle);
        buttonBox.add(new JLabel(""), buttonMiddle);
        buttonBox.add(new JLabel(""), buttonMiddle);
        buttonBox.add(startPositionLabel, buttonBottom);
        buttonBox.add(new JLabel(" - "), buttonBottom);
        buttonBox.add(endPositionLabel, buttonBottom);

        JPanel channelBox = new JPanel(new GridBagLayout());
        GridBagConstraints channelLabelConstraints = new GridBagConstraints();
        channelLabelConstraints.anchor = GridBagConstraints.LINE_START;
        channelLabelConstraints.gridx = 0;
        channelLabelConstraints.insets = insets;
        GridBagConstraints channelSkewConstraints = new GridBagConstraints();
        channelSkewConstraints.anchor = GridBagConstraints.LINE_START;
        channelSkewConstraints.gridx = 1;
        channelSkewConstraints.insets = insets;
        GridBagConstraints channelClipConstraints = new GridBagConstraints();
        channelClipConstraints.anchor = GridBagConstraints.LINE_START;
        channelClipConstraints.gridx = 2;
        channelClipConstraints.insets = insets;
        GridBagConstraints channelBaselimitConstraints = new GridBagConstraints();
        channelBaselimitConstraints.anchor = GridBagConstraints.LINE_START;
        channelBaselimitConstraints.gridx = 3;
        channelBaselimitConstraints.insets = insets;
        GridBagConstraints channelCompensateConstraints = new GridBagConstraints();
        channelCompensateConstraints.anchor = GridBagConstraints.LINE_START;
        channelCompensateConstraints.gridx = 3;
        channelCompensateConstraints.gridwidth = 2;
        channelCompensateConstraints.insets = insets;

        getContentPane().setLayout(new BorderLayout());
        add(buttonBox, BorderLayout.NORTH);
        add(parmBox, BorderLayout.WEST);
        add(channelBox, BorderLayout.EAST);

        setName("ConfigDialog");
        setTitle("ConfigDialog");
        // setPreferredSize(new Dimension(464, 326));

        updateButton.setEnabled(true);
        insertButton.setEnabled(false);
        setEndButton.setEnabled(false);
        deleteButton.setEnabled(false);

        MultiSource mc = p.getMultiSource();
        DataFile datafile = mc.getDataFile();
        Source s = mc.getSource();
        if (datafile != null) {
            nchannel = datafile.getChannels();
            channelLabels = new Label[nchannel];
            skewTextBoxes = new JTextField[nchannel];
            clipTextBoxes = new JTextField[nchannel];
            baseTextBoxes = new JTextField[nchannel];

            channelBox.add(new JLabel("Channel"), channelLabelConstraints);
            channelBox.add(new JLabel("Skew"), channelSkewConstraints);
            channelBox.add(new JLabel("Clip"), channelClipConstraints);
            channelBox.add(new JLabel("Baselimit"), channelBaselimitConstraints);
            for (int i = nchannel - 1; i >= 0; i--) {
                channelLabels[i] = new Label();
                channelLabels[i].setName("channelLabel" + i);
                channelLabels[i].setText(s.getChannelName(i));
                channelLabels[i].setAlignment(Label.CENTER);
                channelBox.add(channelLabels[i], channelLabelConstraints);

                skewTextBoxes[i] = new JTextField();
                skewTextBoxes[i].setName("skewTextBox" + i);
                skewTextBoxes[i].setColumns(2);
                skewTextBoxes[i].setHorizontalAlignment(SwingConstants.RIGHT);
                channelBox.add(skewTextBoxes[i], channelSkewConstraints);

                clipTextBoxes[i] = new JTextField();
                clipTextBoxes[i].setName("clipTextBox" + i);
                clipTextBoxes[i].setColumns(5);
                clipTextBoxes[i].setHorizontalAlignment(SwingConstants.RIGHT);
                channelBox.add(clipTextBoxes[i], channelClipConstraints);

                baseTextBoxes[i] = new JTextField();
                baseTextBoxes[i].setName("baseTextBox" + i);
                baseTextBoxes[i].setColumns(5);
                baseTextBoxes[i].setHorizontalAlignment(SwingConstants.RIGHT);
                channelBox.add(baseTextBoxes[i], channelBaselimitConstraints);
            }
            channelBox.add(compensateCheckbox, channelClipConstraints);

        }
        pack();
        Dimension pdim = getPreferredSize();
        pdim.width = 4 * pdim.width / 3;
        pdim.height += 24;
        setPreferredSize(pdim);
        setSize(pdim);

        rawDataScaleTextbox.setText(Integer.toString(parent.getRawZoom()));
        processedScaleTextbox.setText(Integer.toString(parent.getLumpZoom()));

        if (nchannel > 0) {

            tp = datafile.getTapeParameters().getTapeParameters(parent.getPosition());

            if (tp.getParent() != null) {
                deleteButton.setEnabled(true);
                if (tp.getEnd() == tp.getParent().getEnd()) {
                    setEndButton.setEnabled(true);
                } else if (tp.getStart() < parent.getPosition()) {
                    insertButton.setEnabled(true);
                }
            } else {
                insertButton.setEnabled(true);
            }

            startPositionLabel.setText(Long.toString(tp.getStart()));
            endPositionLabel.setText(Long.toString(tp.getEnd()));
            if (insertButton.isEnabled()) {
                insertPositionLabel.setText(Long.toString(parent.getPosition()));
            } else if (setEndButton.isEnabled()) {
                setEndPositionLabel.setText(Long.toString(parent.getPosition()));
            }

            gainTextbox.setText(Double.toString(tp.getGain()));

            widthTextbox.setText(Integer.toString(tp.getWidth()));

            for (int i = 0; i < nchannel; i++) {
                skewTextBoxes[i].setText(Integer.toString(tp.getSkew()[i]));
                clipTextBoxes[i].setText(Integer.toString(tp.getClip(true, i)));
                baseTextBoxes[i].setText(Integer.toString(tp.getBaseLimit()[i]));
            }

            compensateCheckbox.setSelected(tp.getLowClip());
            compensateCheckbox.setEnabled(tp.canChangeLowClip());
            lowSpaceTextbox.setText(Integer.toString(tp.getLowSpace()));
            highSpaceTextbox.setText(Integer.toString(tp.getHighSpace()));
            crowdTextbox.setText(Integer.toString(tp.getCrowd()));
            gatherTextbox.setText(Integer.toString(tp.getGather()));
            maxSpaceTextbox.setText(Integer.toString(tp.getMaxSpace()));
            minGapTextbox.setText(Integer.toString(tp.getMinGap()));
            maxJunkTextbox.setText(Integer.toString(tp.getMaxJunk()));
        } else {
            compensateCheckbox.setSelected(false);
            gainTextbox.setText("");
            widthTextbox.setText("");
            lowSpaceTextbox.setText("");
            highSpaceTextbox.setText("");
            crowdTextbox.setText("");
            gatherTextbox.setText("");
            maxSpaceTextbox.setText("");
            minGapTextbox.setText("");
            maxJunkTextbox.setText("");
        }
    }

    public void updateValues() {
        parent.setRawZoom(Integer.parseInt(rawDataScaleTextbox.getText()));
        parent.setLumpZoom(Integer.parseInt(processedScaleTextbox.getText()));

        if (tp != null) {
            tp.setGain(Double.parseDouble(gainTextbox.getText()));

            tp.setWidth(Integer.parseInt(widthTextbox.getText()));

            if (compensateCheckbox.isEnabled()) {
                tp.setLowClip(compensateCheckbox.isSelected());
            }

            for (int i = 0; i < nchannel; i++) {
                tp.getSkew()[i] = Integer.parseInt(skewTextBoxes[i].getText());
                tp.setClip(Integer.parseInt(clipTextBoxes[i].getText()), i);
                tp.getBaseLimit()[i] = Integer.parseInt(baseTextBoxes[i].getText());
            }

            tp.setLowSpace(Integer.parseInt(lowSpaceTextbox.getText()));
            tp.setHighSpace(Integer.parseInt(highSpaceTextbox.getText()));
            tp.setCrowd(Integer.parseInt(crowdTextbox.getText()));
            tp.setGather(Integer.parseInt(gatherTextbox.getText()));
            tp.setMaxSpace(Integer.parseInt(maxSpaceTextbox.getText()));
            tp.setMinGap(Integer.parseInt(minGapTextbox.getText()));
            tp.setMaxJunk(Integer.parseInt(maxJunkTextbox.getText()));

            parent.getMultiSource().reEvaluate();
        }
    }

    private Action button1_Click = new AbstractAction("button1_Click") {
                                     public void actionPerformed(final ActionEvent e) {
                                         updateValues();
                                         close();
                                     }
                                 };

    private Action button2_Click = new AbstractAction("button2_Click") {
                                     public void actionPerformed(final ActionEvent e) {
                                         try {
                                             TapeParametersList tpl = parent.getMultiSource().getDataFile().getTapeParameters();
                                             long pos = parent.getPosition();
                                             tp = tpl.add(pos, tpl.getTapeParameters(pos).getEnd());
                                             updateValues();
                                             close();
                                         } catch (Exception _) {
                                         }
                                     }
                                 };

    private Action button3_Click = new AbstractAction("button3_Click") {
                                     public void actionPerformed(final ActionEvent e) {
                                         try {
                                             tp.setEnd(parent.getPosition());
                                             updateValues();
                                             close();
                                         } catch (Exception _) {
                                         }
                                     }
                                 };

    private Action button4_Click = new AbstractAction("button4_Click") {
                                     public void actionPerformed(final ActionEvent e) {
                                         TapeParametersList tpl = parent.getMultiSource().getDataFile().getTapeParameters();
                                         tpl.remove(tp);
                                         close();
                                     }
                                 };

    private void close() {
        setVisible(false);
    }
}
