/**********************************
 *
 * Copyright (C) 2004 Paul Pierce
 *
 * Permission instanceof hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software instanceof furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 *********************************/


package com.piercefuller.a7trk.a7tview;

import java.awt.*;
import javax.swing.*;
import com.piercefuller.a7trk.decode.*;

    /// <summary>
    /// Summary description for ViewScreen.
    /// </summary>
    public class ViewScreen extends JPanel
    {
        public final int SideBorder = 10;
        public final int DataHeight = 16;
        public final int ActivityHeight = 16;
        public final int StatusHeight = 16;
        public final int SliderHeight = 16;

        public int ActivityZoom = 27;

        private A7tView parent;
        private SourceControl[] sourceControl;
        private Colors colors;
        private String lasterr = null;

        public ViewScreen(A7tView p, SourceControl[] sc, Colors colors)
        {
            this.parent = p;
            this.sourceControl = sc;
            this.colors = colors;
        }

        public String getLasterr() {
			return lasterr;
		}

		public void paint(Graphics g)
        {
            try
            {
            	g.setColor(colors.getBackgroundColor());
                g.fillRect(getX(), getY(), getWidth(), getHeight());

                MultiSource multiSource = parent.getMultiSource();
                if (multiSource != null) 
                {
                    Source source = multiSource.getSource();
                
                    int zoom = parent.getZoom();
                    int expansion = 1;
                    if (zoom == 1) 
                    {
                        expansion = 2;
                    }
                    else
                    {
                        zoom = zoom/2;
                    }

                    int screenWidth = this.getWidth() - 2*SideBorder;
                    int screenHeight = this.getHeight();
                    int nchannel = 0;
                    int channelHeight = 0;
                    SourceControl[] boxlevel = null;
                    int[][][] loPointX = null;
                    int[][][] loPointY = null;
                    int[][][] hiPointX = null;
                    int[][][] hiPointY = null;
                    int[] dataValues = null;
                    int[] xDataPoints = null;
                    int[] yDataPoints = null;
                    int dataCount = 0;

                    Color statusPen = colors.getSliderColor();
                    Color[] statusPens = new Color[7];
                    statusPens[0] = Color.white;
                    statusPens[1] = Color.green.darker();
                    statusPens[2] = Color.red;
                    statusPens[3] = Color.green.brighter();
                    statusPens[4] = Color.pink;
                    statusPens[5] = Color.blue;
                    statusPens[6] = Color.orange;

                    int nlevel = 0;
                    nchannel = source.getNumChannels();

                    int sliderTop = screenHeight - SliderHeight;
                    int statusTop = sliderTop;
                    int activityTop = statusTop;
                    if (multiSource.hasActivity()) 
                    {
                        ActivityZoom = 1000;
                        /*
                        if (multiSource instanceof TapeDecoder) 
                        {
                            int gather = ((TapeDecoder)multiSource).getTapeParameters().getGather();
                        }
                        */
                        statusTop = sliderTop - StatusHeight;
                        activityTop = statusTop - ActivityHeight;
                        g.setColor(statusPen);
                        g.drawLine(SideBorder, statusTop + StatusHeight/2-2, screenWidth + SideBorder, statusTop + StatusHeight/2-2);
                        g.drawLine(SideBorder, statusTop + StatusHeight/2+2, screenWidth + SideBorder, statusTop + StatusHeight/2+2);
                    }
                    int dataTop = activityTop;
                    if (multiSource.hasData()) 
                    {
                        dataTop = activityTop - DataHeight;
                        if (zoom == 1) 
                        {
                            dataValues = new int[screenWidth];
                            xDataPoints = new int[screenWidth];
                            yDataPoints = new int[screenWidth];
                        }
                    }

                    channelHeight = (dataTop-20) / source.getNumChannels();

                    nlevel = 1;
                    Source s = source.getSource();
                    for (; s != null; s = s.getSource(), nlevel++);
                    boxlevel = new SourceControl[nlevel];
                    s = source;
                    for (int level = 0; level < nlevel; s = s.getSource(), level++)
                    {
                        boxlevel[level] = null;
                        for (int j = 0; j < sourceControl.length; j++) 
                        {
                            if (sourceControl[j].getCheckBox().getText().equals(s.getName())) 
                            {
                                boxlevel[level] = sourceControl[j];
                                break;
                            }
                        }
                    }

                    loPointX = new int[nlevel][][];
                    loPointY = new int[nlevel][][];
                    hiPointX = new int[nlevel][][];
                    hiPointY = new int[nlevel][][];
                    for (int level = 0; level < nlevel; level++) 
                    {
                        loPointX[level] = new int[nchannel+2][];
                        loPointY[level] = new int[nchannel+2][];
                        hiPointX[level] = new int[nchannel+2][];
                        hiPointY[level] = new int[nchannel+2][];
                        for (int i = 0; i < nchannel+2; i++) 
                        {
                            loPointX[level][i] = new int[screenWidth/expansion];
                            loPointY[level][i] = new int[screenWidth/expansion];
                            hiPointX[level][i] = new int[screenWidth/expansion];
                            hiPointY[level][i] = new int[screenWidth/expansion];
                            for (int j = 0; j < screenWidth/expansion; j++) 
                            {
                                loPointX[level][i][j] = expansion*j + SideBorder;
                                loPointY[level][i][j] = 0;
                                hiPointX[level][i][j] = expansion*j + SideBorder;
                                hiPointY[level][i][j] = 0;
                            }
                        }
                    }

                    long spos = parent.getPosition() - parent.getZoomWidth()/2;
                    long epos = spos + parent.getZoomWidth();

                    multiSource.setPosition(spos, epos - spos);
                    int[][] newMin = new int[nlevel][];
                    int[][] newMax = new int[nlevel][];
                    for (int level = 0; level < nlevel; level++) 
                    {
                        newMin[level] = new int[nchannel+2];
                        newMax[level] = new int[nchannel+2];
                        for (int ch = 0; ch < nchannel+2; ch++) 
                        {
                            newMin[level][ch] = 0;
                            newMax[level][ch] = 0;
                        }
                    }

                    int sn = 0;
                    int statusStart = 0;
                    int statusPos = 0;
                    for (int x = 0; x < screenWidth/expansion; x++) 
                    {
                        for (int level = 0; level < nlevel; level++) 
                        {
                            for (int ch = 0; ch < nchannel; ch++) 
                            {
                                newMin[level][ch] = channelHeight/2;
                                newMax[level][ch] = -channelHeight/2;
                            }
                        }
                        if (multiSource.hasActivity())
                        {
                            for (int ch = nchannel; ch < nchannel+2; ch++) 
                            {
                                newMin[0][ch] = ActivityHeight;
                                newMax[0][ch] = 0;
                            }
                        }
                        for (int i = 0; i < zoom; i++) 
                        {
                            for (int level = 0; level < nlevel; level++) 
                            {
                                int[] samples = source.getSamples(level);
                                for (int ch = 0; ch < nchannel; ch++) 
                                {
                                    int v = samples[ch]/(2*boxlevel[level].getZoom() / channelHeight);

                                    if (v < newMin[level][ch]) 
                                    {
                                        if (v > -channelHeight/2) 
                                        {
                                            newMin[level][ch] = v;
                                        }
                                        else
                                        {
                                            newMin[level][ch] = -channelHeight/2;
                                        }
                                    }
                                    if (v > newMax[level][ch]) 
                                    {
                                        if (v < channelHeight/2) 
                                        {
                                            newMax[level][ch] = v;
                                        }
                                        else
                                        {
                                            newMax[level][ch] = channelHeight/2;
                                        }
                                    }
                                }
                            }

                            if (multiSource.hasActivity())
                            {
                                for (int ch = nchannel; ch < nchannel+2; ch++) 
                                {
                                    int v;
                                    if (ch == nchannel) 
                                    {
                                        v = ActivityHeight*multiSource.getActivityCountHere()/(nchannel);
                                    }
                                    else
                                    {
                                        v = ActivityHeight*multiSource.getActivityLumpedHere()/ActivityZoom;
                                    }

                                    if (v < newMin[0][ch]) 
                                    {
                                        if (v > -channelHeight/2) 
                                        {
                                            newMin[0][ch] = v;
                                        }
                                        else
                                        {
                                            newMin[0][ch] = -channelHeight/2;
                                        }
                                    }
                                    if (v > newMax[0][ch]) 
                                    {
                                        if (v < channelHeight/2) 
                                        {
                                            newMax[0][ch] = v;
                                        }
                                        else
                                        {
                                            newMax[0][ch] = channelHeight/2;
                                        }
                                    }
                                }

                                int status = multiSource.getActivityStatusHere();
                                if (status != sn) 
                                {
                                    if (sn != 0) 
                                    {
                                        int sX = SideBorder + statusStart*expansion/zoom;
                                        int eX = SideBorder + (statusPos-1)*expansion/zoom;
                                        g.setColor(statusPens[sn]);
                                        g.drawLine(sX, statusTop + StatusHeight/2-1, eX, statusTop + StatusHeight/2-1);
                                        g.drawLine(sX, statusTop + StatusHeight/2, eX, statusTop + StatusHeight/2);
                                        g.drawLine(sX, statusTop + StatusHeight/2+1, eX, statusTop + StatusHeight/2+1);
                                    }
                                    statusStart = statusPos;
                                    sn = status;
                                }
                                statusPos++;
                            }

                            if (dataValues != null && multiSource.isDataValidHere())
                            {
                                dataValues[dataCount] = multiSource.getDataHere();
                                xDataPoints[dataCount] = loPointX[0][0][x] - g.getFont().getSize();
                                yDataPoints[dataCount] = dataTop;
                                dataCount++;
                            }

                            multiSource.advance();
                        }

                        for (int level = 0; level < nlevel; level++) 
                        {
                            for (int ch = 0; ch < nchannel; ch++) 
                            {
                                loPointY[level][ch][x] = 10 + channelHeight*(6-ch) + channelHeight/2 - newMin[level][ch];
                                hiPointY[level][ch][x] = 10 + channelHeight*(6-ch) + channelHeight/2 - newMax[level][ch];
                            }
                        }
                        if (multiSource.hasActivity())
                        {
                            for (int ch = nchannel; ch < nchannel+2; ch++) 
                            {
                                loPointY[0][ch][x] = activityTop + ActivityHeight - newMin[0][ch];
                                hiPointY[0][ch][x] = activityTop + ActivityHeight - newMax[0][ch];
                            }
                        }
                    }

                    multiSource.setPosition(parent.getPosition());
                    if (parent.centerEnabled()) 
                    {
                    	g.setColor(colors.getCenterColor());
                        g.drawLine(SideBorder + screenWidth/2, 0, SideBorder + screenWidth/2, screenHeight-1);
                    }
                    if (parent.zeroEnabled()) 
                    {
                        g.setColor(colors.getZeroColor());
                        for (int ch = 0; ch < nchannel; ch++) 
                        {
                            g.drawLine(SideBorder, 10 + channelHeight*(6-ch) + channelHeight/2, SideBorder + screenWidth, 10 + channelHeight*(6-ch) + channelHeight/2);
                        }
                        g.drawLine(SideBorder, activityTop + ActivityHeight, SideBorder + screenWidth, activityTop + ActivityHeight);
                    }
                    if (parent.clipEnabled() && multiSource instanceof TapeDecoder) 
                    {
                    	g.setColor(colors.getClipColor());
                        for (int ch = 0; ch < nchannel; ch++) 
                        {
                            int v = ((TapeDecoder)multiSource).getTapeParameters().getClip(true, ch) / (2*sourceControl[1].getZoom() / channelHeight);
                            if (v < channelHeight/2) 
                            {
                                g.drawLine(SideBorder, 10 + channelHeight*(6-ch) + channelHeight/2 + v, SideBorder + screenWidth, 10 + channelHeight*(6-ch) + channelHeight/2 + v);
                                g.drawLine(SideBorder, 10 + channelHeight*(6-ch) + channelHeight/2 - v, SideBorder + screenWidth, 10 + channelHeight*(6-ch) + channelHeight/2 - v);
                            }
                        }
                    }

                    if (multiSource.hasActivity())
                    {
                        Color activityCountPen = null;
                        Color activityLumpedPen = null;
                        for (int level = nlevel-1; level >= 0; level--)
                        {
                            Color dataPen = boxlevel[level].getColor();
                            if (level == nlevel-1)
                            {
                                activityCountPen = dataPen;
                            }
                            if (level == nlevel-2)
                            {
                                activityLumpedPen = dataPen;
                            }
                            g.setColor(dataPen);
                            for (int ch = 0; ch < nchannel; ch++) 
                            {
                                if (boxlevel[level].getCheckBox().isSelected()) 
                                {
                                    g.drawPolyline(loPointX[level][ch], loPointY[level][ch], loPointX[level][ch].length);
                                    g.drawPolyline(hiPointX[level][ch], hiPointY[level][ch], hiPointX[level][ch].length);
                                }
                            }
                        }
                        g.setColor(activityCountPen);
                        g.drawPolyline(loPointX[0][nchannel], loPointY[0][nchannel], loPointX[0][nchannel].length);
                        g.setColor(activityLumpedPen);
                        g.drawPolyline(hiPointX[0][nchannel+1], hiPointY[0][nchannel+1], hiPointX[0][nchannel].length);
                    }

                    if (dataValues != null) 
                    {
                        for (int i = 0; i < dataCount; i++) 
                        {
                        	g.drawString(Integer.toHexString(dataValues[i]), xDataPoints[i], yDataPoints[i]);
                        }
                    }

                    int sliderX = SideBorder + (int)(screenWidth*(parent.getPosition())/source.getLength());
                    int sposX = SideBorder + (int)(screenWidth*spos/source.getLength());
                    int eposX = SideBorder + (int)(screenWidth*epos/source.getLength());
                    int svposX = SideBorder + (int)(screenWidth*source.getStart()/source.getLength());
                    int evposX = SideBorder + (int)(screenWidth*source.getEnd()/source.getLength());
                    g.setColor(colors.getSliderColor());
                    g.drawLine(SideBorder, sliderTop + SliderHeight/2-1, screenWidth + SideBorder, sliderTop + SliderHeight/2-1);
                    g.drawLine(SideBorder, sliderTop + SliderHeight/2+1, screenWidth + SideBorder, sliderTop + SliderHeight/2+1);
                    g.setColor(colors.getCenterColor());
                    g.drawLine(svposX, sliderTop + SliderHeight/2, evposX, sliderTop + SliderHeight/2);
                    g.setColor(colors.getSliderColor());
                    g.drawLine(sposX, sliderTop + SliderHeight/2, eposX, sliderTop + SliderHeight/2);
                    g.drawLine(sliderX, sliderTop + 2, sliderX, sliderTop + SliderHeight - 2);
                }
            }
            catch (Exception ex)
            {
                lasterr = ex.toString();
            }
        }
    }
