/**********************************
 *
 * Copyright (C) 2004 Paul Pierce
 *
 * Permission instanceof hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software instanceof furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 *********************************/


package com.piercefuller.a7trk.decode;

import java.io.*;

    /// <summary>
    /// Summary description for Record.
    /// </summary>
    public class Record
    {
        private int number;
        private long start;
        private long end;
        private int count = 0;
        private int errors = 0;
        private int fixups = 0;
        private boolean motionForward = true;

        public Record(int number, long start, long end, boolean motionForward)
        {
            this.number = number;
            this.start = start;
            this.end = end;
            this.motionForward = motionForward;
            end = start;
        }

        public int getNumber() { return number; }
        public long getStart() { return start; }
        public long getEnd() { return end; }
        public int getCount() { return count; }
        public void setCount(int v) { count = v; }
        public int getErrors() { return errors; }
        public void setErrors(int errors) { this.errors = errors; }
        public int getFixups() { return fixups; }
        public void setFixups(int fixups) { this.fixups = fixups; }
        public boolean getMotionForward() { return motionForward; }
        public void setMotionForward(boolean mf) { motionForward = mf; }

        public void write(BufferedWriter ow, int indent) throws IOException
        {
            String errs = "";
            if (errors > 0) 
            {
                errs = " errors=\"" + errors + "\"";
            }
            if (fixups > 0) 
            {
                errs += " fixups=\"" + fixups + "\"";
            }
            String inds = "                    ".substring(0, indent);
            ow.write(inds + "<record start=\"" + start + "\" end=\"" + end + "\"" + errs + " />");
            ow.newLine();
        }
    }
