/**
 * 
 */
package org.avparade.xml;

import java.util.Calendar;

public class IsoDuration implements Comparable<IsoDuration> {
	private boolean forward = true;
	private int months = 0;
	private int days = 0;
	private long millis = 0;

	/**
	 * 
	 */
	public IsoDuration() {
	}

	/**
	 * @param millis
	 */
	public IsoDuration(long millis) {
		this.millis = millis;
		normalize();
	}

	/**
	 * @param months
	 * @param days
	 * @param millis
	 */
	public IsoDuration(int months, int days, long millis) {
		this.months = months;
		this.days = days;
		this.millis = millis;
		normalize();
	}

	/**
	 * @param years
	 * @param months
	 * @param days
	 * @param hours
	 * @param minutes
	 * @param seconds
	 */
	public IsoDuration(int years, int months, int days, int hours, int minutes, double seconds) {
		this.months = 12*years + months;
		this.days = days;
		this.millis = Math.round(1000*(seconds + 60*(minutes + 60*hours)));
		normalize();
	}

	/**
	 * 
	 * @param startTime
	 * @param endTime
	 */
	public IsoDuration(IsoTime startTime, IsoTime endTime) {
		if (startTime.compareTo(endTime) > 0) {
			IsoTime t = startTime;
			startTime = endTime;
			endTime = t;
			forward = false;
		}
		Calendar startCal = startTime.getCalendar();
		Calendar endCal = endTime.getCalendar();
		months = 12*(endCal.get(Calendar.YEAR) - startCal.get(Calendar.YEAR));
		months += endCal.get(Calendar.MONTH) - startCal.get(Calendar.MONTH);
		days = endCal.get(Calendar.DAY_OF_MONTH) - startCal.get(Calendar.DAY_OF_MONTH);
		millis = 1000*60*60*(endCal.get(Calendar.HOUR_OF_DAY) - startCal.get(Calendar.HOUR_OF_DAY));
		millis += 1000*60*(endCal.get(Calendar.MINUTE) - startCal.get(Calendar.MINUTE));
		millis += 1000*(endCal.get(Calendar.SECOND) - startCal.get(Calendar.SECOND));
		millis += endCal.get(Calendar.MILLISECOND) - startCal.get(Calendar.MILLISECOND);
		if (millis < 0) {
			days -= 1;
			millis += 1000*60*60*24;
		}
		if (days < 0) {
			days = startCal.getActualMaximum(Calendar.DAY_OF_MONTH) - startCal.get(Calendar.DAY_OF_MONTH) + endCal.get(Calendar.DAY_OF_MONTH);
			months -= 1;
		}
	}

	private static final char[] procession = "YMDHM.S".toCharArray();

	/**
	 * 
	 * @param iso
	 * @param forward
	 */
	public IsoDuration(String iso, boolean forward) {
		this.forward = forward;
		Exception ex = null;
		try {
			for (;;) {	// Not a loop, just something to break out of
				if (Character.toUpperCase(iso.charAt(0)) != 'P') {
					break;
				}
				char c;
				int i = 1;
				int p = 0;
				boolean decimal = false;
				for (; i < iso.length(); ) {
					c = Character.toUpperCase(iso.charAt(i));
					if (p <= 3 && c == 'T') {
						p = 3;
						++i;
					}
					int t = 0;
					int m = 1;
					for (; i+1 < iso.length(); ++i) {
						c = Character.toUpperCase(iso.charAt(i));
						if (c >= '0' && c <= '9') {
							t = 10*t + (c - '0');
							m *= 10;
							continue;
						}
						break;
					}
					if (m == 1) {
						break;
					}
					c = Character.toUpperCase(iso.charAt(i++));
					for (; p < procession.length; p++) {
						if (c == procession[p]) {
							break;
						}
					}
					if (p == procession.length) {
						break;
					}
					switch (p) {
					case 0:
						addYears(t);
						break;
					case 1:
						addMonths(t);
						break;
					case 2:
						addDays(t);
						break;
					case 3:
						addHours(t);
						break;
					case 4:
						addMinutes(t);
						break;
					case 5:
						decimal = true;
						addSeconds(t);
						break;
					case 6:
						if (decimal) {
							addSeconds((double)t/(double)m);
						} else {
							addSeconds(t);
						}
						break;
					}
					++p;
				}
				return;
			}
		} catch (RuntimeException e) {
			ex = e;
		}
		NumberFormatException em = new NumberFormatException("Incompatible ISO time string <" + iso + ">");
		if (ex != null) {
			em.initCause(ex);
		}
		throw em;
	}

	/**
	 * @return Returns the years.
	 */
	public int getYears() {
		return months / 12;
	}

	/**
	 * @param years The years to set.
	 */
	public void addYears(int years) {
		this.months += 12 * years;
		normalize();
	}

	/**
	 * @return Returns the months.
	 */
	public int getMonths() {
		return months % 12;
	}

	/**
	 * @param months The months to set.
	 */
	public void addMonths(int months) {
		this.months += months;
		normalize();
	}

	/**
	 * @return Returns the days.
	 */
	public int getDays() {
		return days;
	}

	/**
	 * @param days The days to set.
	 */
	public void addDays(int days) {
		this.days += days;
		normalize();
	}

	/**
	 * @return Returns the hours.
	 */
	public int getHours() {
		return (int)(millis / (1000 * 60 * 60));
	}

	/**
	 * @param hours The hours to set.
	 */
	public void addHours(int hours) {
		millis += 1000 * 60 * 60 * hours;
		normalize();
	}

	/**
	 * @return Returns the minutes.
	 */
	public int getMinutes() {
		return (int)((millis / (1000 * 60)) % 60);
	}

	/**
	 * @param minutes The minutes to set.
	 */
	public void addMinutes(int minutes) {
		millis += 1000 * 60 * minutes;
		normalize();
	}

	/**
	 * @return Returns the seconds.
	 */
	public double getSeconds() {
		return (millis % 60*1000) / 1000.0;
	}

	/**
	 * @param seconds The seconds to set.
	 */
	public void addSeconds(double seconds) {
		millis += 1000 * seconds;
		normalize();
	}

	/**
	 * @return Returns true if duration is forward in time.
	 */
	public boolean isForward() {
		return forward;
	}

	/**
	 * @param forward True for forward in time.
	 */
	public void setForward(boolean forward) {
		this.forward = forward;
	}

	/**
	 * @return Returns the time component (hours, minutes, seconds) in milliseconds.
	 */
	public long getMillis() {
		return millis;
	}

	/**
	 * @return Returns approximate (since start time is not known) total duration in milliseconds.
	 */
	public long getApproximateTotalMillis() {
		return millis + 1000L * 60 * 60 * 24 * (days + (3*365 + 366 + 2*12) * months / (4*12));
	}
	
	/**
	 * 
	 * @param startTime
	 * @return Returns total duration in milliseconds, assuming the given start time.
	 */
	public long getTotalMillis(IsoTime startTime) {
		return getEndTime(startTime).getMillis() - startTime.getMillis();
	}

	/**
	 * 
	 * @param startTime
	 * @return Returns the time following the given start time by this duration.
	 */
	public IsoTime getEndTime(IsoTime startTime) {
		return getEndTime(startTime, forward);
	}

	/**
	 * 
	 * @param startTime
	 * @param forward Specify the direction explicitly.
	 * @return Returns the time following the given start time by this duration.
	 */
	public IsoTime getEndTime(IsoTime startTime, boolean forward) {
		Calendar c = startTime.getCalendar();
		if (forward) {
			c.add(Calendar.MONTH, months);
			c.add(Calendar.DATE, days + (int)(millis / (1000*60*60*24)));
			c.add(Calendar.SECOND, (int)((millis % (1000*60*60*24)) / 1000));
			c.add(Calendar.MILLISECOND, (int)(millis % 1000));
		} else {
			c.add(Calendar.MONTH, -months);
			c.add(Calendar.DATE, -(days + (int)(millis / (1000*60*60*24))));
			c.add(Calendar.SECOND, -(int)((millis % (1000*60*60*24)) / 1000));
			c.add(Calendar.MILLISECOND, -(int)(millis % 1000));
		}
		return new IsoTime(c);
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	
	public boolean equals(Object obj) {
		if (obj instanceof IsoDuration) {
			return compareTo((IsoDuration)obj) == 0;
		}
		return false;
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	
	public int compareTo(IsoDuration o) {
		if (months == o.months) {
			if (days == o.days) {
				if (millis > o.millis) {
					return 1;
				}
				if (millis < o.millis) {
					return -1;
				}
				return 0;
			}
			if (millis == o.millis) {
				if (days > o.days) {
					return 1;
				} if (days < o.days) {
					return -1;
				}
				return 0;
			}
		}
		if (days == o.days && millis == o.millis) {
			if (months > o.months) {
				return 1;
			}
			if (months < o.months) {
				return -1;
			}
			return 0;
		}
		long myTotal = getApproximateTotalMillis();
		long oTotal = o.getApproximateTotalMillis();
		if (myTotal > oTotal) {
			return 1;
		}
		if (myTotal < oTotal) {
			return -1;
		}
		return 0;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	
	public String toString() {
		StringBuffer sb = new StringBuffer("P");
		int t;
		
		t = getYears();
		if (t != 0) {
			sb.append(t + "Y");
		}
		
		t = getMonths();
		if (t != 0) {
			sb.append(t + "M");
		}

		t = getDays();
		if (t != 0) {
			sb.append(t + "D");
		}

		if (millis != 0) {
			sb.append('T');

			t = getHours();
			if (t != 0) {
				sb.append(t + "H");
			}

			t = getMinutes();
			if (t != 0) {
				sb.append(t + "M");
			}

			double s = getSeconds();
			if (s != 0) {
				int s1 = (int)Math.floor(s);
				int s2 = (int)(millis % 1000);
				sb.append(s1);
				if (s2 != 0) {
					sb.append('.');
					sb.append(s2);
				}
				sb.append('S');
			}
		}
		return sb.toString();
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	
	public Object clone() {
		return new IsoDuration(months, days, millis);
	}

	private void normalize() {
		if (getApproximateTotalMillis() < 0) {
			forward = !forward;
			months = -months;
			days = -days;
			millis = -millis;
		}
	}
}