/**
 * 
 */
package org.avparade.xml;

import java.util.*;
import java.text.*;


/**
 * @author prp
 *
 */
public class IsoTimeTest {

	private static final int TimeFrac        = 0x00000001;
	private static final int TimeSecond      = 0x00000002 | TimeFrac;
	private static final int TimeMinute      = 0x00000004;
	private static final int TimeHour        = 0x00000008;
	private static final int TimeDay         = 0x00000010;
	private static final int TimeMonth       = 0x00000020;
	private static final int TimeYear        = 0x00000040;
	private static final int TimeMask        = TimeSecond | TimeMinute | TimeHour | TimeDay | TimeMonth | TimeYear;

	private static final int DurationSecond  = 0x00000080;
	private static final int DurationMinute  = 0x00000100;
	private static final int DurationHour    = 0x00000200;
	private static final int DurationDay     = 0x00000400;
	private static final int DurationMonth   = 0x00000800;
	private static final int DurationYear    = 0x00001000;
	private static final int DurationWeek    = 0x00002000;
	private static final int DurationNotWeek = DurationSecond | DurationMinute | DurationHour | DurationDay | DurationMonth | DurationYear;
	private static final int DurationMask    = DurationNotWeek | DurationWeek;
	
	private static final int Local           = 0x00004000;
	private static final int Punctuation     = 0x00008000;
	private static final int LowerCase       = 0x00010000;
	private static final int Order           = 0x00020000;
	private static final int Separator       = 0x00040000;
	private static final int MiscMask        = Local | Punctuation | LowerCase | Order | Separator;

	private static final int SelectMask      = TimeMask | DurationMask | MiscMask;

	private static final int DMaxSecond      = 76;
	private static final int DMaxMinute      = 76;
	private static final int DMaxHour        = 37;
	private static final int DMaxDay         = 61;
	private static final int DMaxMonth       = 15;
	private static final int DMaxYear        = 5;

	private static final int[] DaysInMonth = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
	
	private int select;
	private int timeValue;
	private int durationValue;
	
	private int year;
	private int month;
	private int day;
	private int hour;
	private int minute;
	private int second;
	private int millisecond;

	private int years = 0;
	private int months = 0;
	private int days = 0;
	private int hours = 0;
	private int minutes = 0;
	private int seconds = 0;
	
	public IsoTimeTest(String s) {
		int p1 = s.indexOf('-');
		if (p1 >= 0) {
			s = s.substring(p1 + 1);
		}
		p1 = s.indexOf('=');
		if (p1 > 0) {
			s = s.substring(0, p1);
		}
		p1 = s.indexOf('.');
		int p2 = s.indexOf('.', p1 + 1);
		int p3 = s.indexOf('.', p2 + 1);
		select = Integer.parseInt(s.substring(0, p1));
		timeValue = Integer.parseInt(s.substring(p1 + 1, p2));
		millisecond = Integer.parseInt(s.substring(p2 + 1, p3));
		durationValue = Integer.parseInt(s.substring(p3 + 1));
		setFields();
	}

	/**
	 * @param select
	 * @param timeValue
	 * @param milliValue
	 * @param durationValue
	 */
	public IsoTimeTest(int select, int timeValue, int milliValue, int durationValue) {
		this.select = select;
		this.timeValue = timeValue;
		this.millisecond = milliValue;
		this.durationValue = durationValue;
		setFields();
	}
	
	private void setFields() {
		select = select & SelectMask;
		timeValue = (timeValue & Integer.MAX_VALUE) % (2100*12*31*25*60*61);
		millisecond = (millisecond & Integer.MAX_VALUE) % 1000;
		durationValue = (durationValue & Integer.MAX_VALUE) % (DMaxYear*DMaxMonth*DMaxDay*DMaxHour*DMaxMinute*DMaxSecond);

		if ((select & (TimeYear | TimeMonth | TimeDay)) != 0) {
			select |= TimeYear;
		}
		if ((select & (TimeHour | TimeMinute | TimeSecond)) != 0) {
			select |= TimeHour;
		}

		if ((select & (TimeHour | TimeMinute | TimeSecond)) != 0) {
			if ((select & TimeDay) == 0) {
				select &= ~(TimeYear | TimeMonth);
			} else {
				select |= TimeHour;
			}
			if ((select & (TimeHour | TimeSecond)) > TimeHour) {
				select |= TimeMinute;
			} else if ((select & (TimeHour | TimeMinute | TimeSecond)) == (TimeHour | TimeMinute)) {
				select |= Punctuation;
			}
		} else {
			select &= ~Local;
			select |= TimeYear;
		}
		if ((select & (TimeYear | TimeDay)) == (TimeYear | TimeDay)) {
			select |= TimeMonth;
		} else if ((select & (TimeYear | TimeMonth | TimeDay)) == (TimeYear | TimeMonth)) {
			select |= Punctuation;
		}

		
		if ((select & DurationNotWeek) != 0) {
			select &= ~DurationWeek;
		}
		if ((select & TimeYear) == 0) {
			select &= ~DurationYear;
			if ((select & TimeMonth) == 0) {
				select &= ~DurationMonth;
				if ((select & TimeDay) == 0) {
					select &= ~(DurationDay | DurationWeek);
					if ((select & TimeHour) == 0) {
						select &= ~DurationHour;
						if ((select & TimeMinute) == 0) {
							select &= ~DurationMinute;
							if ((select & TimeSecond) == 0) {
								select &= ~DurationSecond;
							}
						}
					}
				}
			}
		}
		if ((select & TimeSecond) == 0) {
			select &= ~DurationSecond;
			if ((select & TimeMinute) == 0) {
				select &= ~DurationMinute;
				if ((select & TimeHour) == 0) {
					select &= ~DurationHour;
					if ((select & TimeDay) == 0) {
						select &= ~(DurationDay | DurationWeek);
						if ((select & TimeMonth) == 0) {
							select &= ~DurationMonth;
							if ((select & TimeYear) == 0) {
								select &= ~DurationYear;
							}
						}
					}
				}
			}
		}

		year = ((timeValue / (12*31*25*60*61)) % 2100) + 1900;
		month = (timeValue / (31*25*60*61) % 12) + 1;
		day = (timeValue / (25*60*61)) % 31;
		int mdays = DaysInMonth[month - 1];
		if (month == 2 && (year % 4) == 0 && (year % 100) != 0) {
			mdays++;
		}
		day = (day % mdays) + 1;
		hour = (timeValue / (60*61)) % 25;
		minute = (timeValue / (61)) % 60;
		second = timeValue % 61;

		if ((select & DurationYear) != 0) {
			years = (durationValue / (DMaxMonth*DMaxDay*DMaxHour*DMaxMinute*DMaxSecond)) % DMaxYear;
		}
		if ((select & DurationMonth) != 0) {
			months = (durationValue / (DMaxDay*DMaxHour*DMaxMinute*DMaxSecond)) % DMaxMonth;
		}
		if ((select & DurationWeek) != 0) {
			days = 7*((durationValue / (DMaxHour*DMaxMinute*DMaxSecond)) % 104);
		}
		if ((select & DurationDay) != 0) {
			days += (durationValue / (DMaxHour*DMaxMinute*DMaxSecond)) % DMaxDay;
		}
		if ((select & DurationHour) != 0) {
			hours = (durationValue / (DMaxMinute*DMaxSecond)) % DMaxHour;
		}
		if ((select & DurationMinute) != 0) {
			minutes = (durationValue / DMaxSecond) % DMaxMinute;
		}
		if ((select & DurationSecond) != 0) {
			seconds = durationValue % DMaxSecond;
		}
	}

	public Calendar getStart() {
		return getTime(((select & Order) != 0) ? 0 : -1);
	}

	public Calendar getEnd() {
		return getTime(((select & Order) != 0) ? 1 : 0);
	}

	private Calendar getTime(int d) {
		Calendar calendar;
		if ((select & Local) != 0) {
			calendar = Calendar.getInstance();
		} else {
			calendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
		}
		boolean clear = false;
		if ((select & TimeYear) != 0) {
			calendar.set(Calendar.YEAR, year);
			clear = true;
		}
		if ((select & TimeMonth) != 0) {
			calendar.set(Calendar.MONTH, month);
			clear = true;
		} else if (clear) {
			calendar.set(Calendar.MONTH, Calendar.JANUARY);
		}
		if ((select & TimeDay) != 0) {
			calendar.set(Calendar.DAY_OF_MONTH, day);
			clear = true;
		} else if (clear) {
			calendar.set(Calendar.DAY_OF_MONTH, 1);
		}
		if ((select & TimeHour) != 0) {
			calendar.set(Calendar.HOUR_OF_DAY, hour);
			clear = true;
		} else if (clear) {
			calendar.set(Calendar.HOUR_OF_DAY, 0);
		}
		if ((select & TimeMinute) != 0) {
			calendar.set(Calendar.MINUTE, minute);
			clear = true;
		} else if (clear) {
			calendar.set(Calendar.MINUTE, 0);
		}
		if ((select & TimeSecond) != 0) {
			calendar.set(Calendar.SECOND, second);
		} else if (clear) {
			calendar.set(Calendar.SECOND, 0);
		}
		if ((select & TimeFrac) != 0) {
			calendar.set(Calendar.MILLISECOND, millisecond);
		} else {
			calendar.set(Calendar.MILLISECOND, 0);
		}
		if (d != 0) {
			calendar.add(Calendar.YEAR, d*years);
			calendar.add(Calendar.MONTH, d*months);
			calendar.add(Calendar.DAY_OF_MONTH, d*days);
			calendar.add(Calendar.HOUR_OF_DAY, d*hours);
			calendar.add(Calendar.MINUTE, d*minutes);
			calendar.add(Calendar.SECOND, d*seconds);
		}
		return calendar;
	}
	
	public String getString() {
		StringBuffer sb = new StringBuffer();
		if ((select & Order) != 0) {
			if ((select & TimeMask) != 0) {
				addTime(sb);
			}
			if ((select & DurationMask) != 0) {
				if ((select & TimeMask) != 0) {
					if ((select & Separator) != 0) {
						sb.append('/');
					} else {
						sb.append("--");
					}
				}
				addDuration(sb);
			}
		} else {
			if ((select & DurationMask) != 0) {
				addDuration(sb);
				if ((select & TimeMask) != 0) {
					if ((select & Separator) != 0) {
						sb.append('/');
					} else {
						sb.append("--");
					}
				}
			}
			if ((select & TimeMask) != 0) {
				addTime(sb);
			}
		}
		String s = sb.toString();
		if ((select & LowerCase) != 0) {
			s = s.toLowerCase();
		}
		return s;
	}
	
	private void addTime(StringBuffer sb) {
		boolean going = false;
		boolean punctuation = (select & Punctuation) != 0;
		if ((select & TimeYear) != 0) {
			sb.append(Integer.toString(10000 + year).substring(1));
			going = true;
		}
		if ((select & TimeMonth) != 0) {
			if (going && punctuation) {
				sb.append('-');
			}
			sb.append(Integer.toString(101 + month).substring(1));
			going = true;
		}
		if ((select & TimeDay) != 0) {
			if (going && punctuation) {
				sb.append('-');
			}
			sb.append(Integer.toString(100 + day).substring(1));
			going = true;
		}
		if ((select & (TimeHour | TimeMinute | TimeSecond)) == 0) {
			return;
		}
		if (going) {
			sb.append('T');
			going = false;
		}
		if ((select & TimeHour) != 0) {
			sb.append(Integer.toString(100 + hour).substring(1));
			going = true;
		}
		if ((select & TimeMinute) != 0) {
			if (going && punctuation) {
				sb.append(':');
			}
			sb.append(Integer.toString(100 + minute).substring(1));
			going = true;
		}
		if ((select & TimeSecond) != 0) {
			if (going && punctuation) {
				sb.append(':');
			}
			sb.append(Integer.toString(100 + second).substring(1));
		}
		if ((select & TimeFrac) != 0) {
			sb.append('.');
			int frac = 1000 + millisecond;
			while (frac > 10 && frac % 10 == 0) {
				frac /= 10;
			}
			sb.append(Integer.toString(frac).substring(1));
		}
		if ((select & Local) == 0) {
			sb.append('Z');
		}
		going = true;
	}
	
	private void addDuration(StringBuffer sb) {
		sb.append('P');
		if ((select & DurationYear) != 0) {
			sb.append(Integer.toString(years));
			sb.append('Y');
		}
		if ((select & DurationMonth) != 0) {
			sb.append(Integer.toString(months));
			sb.append('M');
		}
		if ((select & DurationWeek) != 0) {
			sb.append(Integer.toString(days / 7));
			sb.append('W');
		}
		if ((select & DurationDay) != 0) {
			sb.append(Integer.toString(days));
			sb.append('D');
		}
		if ((select & (DurationHour | DurationMinute | DurationSecond)) != 0) {
			sb.append('T');
			if ((select & DurationHour) != 0) {
				sb.append(Integer.toString(hours));
				sb.append('H');
			}
			if ((select & DurationMinute) != 0) {
				sb.append(Integer.toString(minutes));
				sb.append('M');
			}
			if ((select & DurationSecond) != 0) {
				sb.append(Integer.toString(seconds));
				sb.append('S');
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	
	public String toString() {
		return "IsoTimeTest-" + select + "." + timeValue + "." + millisecond + "." + durationValue + "=" + getString();
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		Random random = new Random();
		int pass = 0;
		int fail = 0;

		DateFormat fmt = DateFormat.getDateTimeInstance();
		fmt.setCalendar(Calendar.getInstance(TimeZone.getTimeZone("UTC")));
		int n = args.length;
		if (n == 0) {
			n = 1000;
		}
		for (int i = 0; i < n; i++) {
			IsoTimeTest test;
			if (i < args.length) {
				test = new IsoTimeTest(args[i]);
			} else {
				test = new IsoTimeTest(random.nextInt(Integer.MAX_VALUE), random.nextInt(Integer.MAX_VALUE), random.nextInt(Integer.MAX_VALUE), random.nextInt(Integer.MAX_VALUE));
			}
			try {
				IsoTime t = new IsoTime(test.getString());
				Date start = t.getStart().getTime();
				Date end = t.getEnd().getTime();
				Date testStart = test.getStart().getTime();
				Date testEnd = test.getEnd().getTime();
				if (!start.equals(testStart)) {
					++fail;
					System.err.println(test.toString() + "  start " + fmt.format(start) + " != " + fmt.format(testStart));
				} else if (!end.equals(testEnd)) {
					++fail;
					System.err.println(test.toString() + "   end  " + fmt.format(end) + " != " + fmt.format(testEnd));
				} else {
					++pass;
					//System.out.println(test.getString());
				}
			} catch (NumberFormatException e) {
				++fail;
				System.err.println(test.toString() + " error " + e);
			} catch (RuntimeException e) {
				++fail;
				System.err.println(test.toString());
				e.printStackTrace();
			}
		}
		System.out.println("Pass " + pass + " Fail " + fail);
	}
}
