/**
 * JaxpXMLHandler.java
 *
 * Copyright (C) 2007 Paul R. Pierce
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see http://www.gnu.org/licenses/gpl.txt
 */
package org.avparade.xml;

import java.io.*;
import org.w3c.dom.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

/**
 * org.avparade.xml.JaxpXMLHandler
 *
 */
public class JaxpXMLHandler extends XMLHandlerBase {
	//private DocumentBuilder builder;
	private Transformer transformer;
	
	/**
	 * 
	 */
	public JaxpXMLHandler() throws Exception {
		//builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		transformer = TransformerFactory.newInstance().newTransformer();
	}

	/* (non-Javadoc)
	 * @see org.avparade.xml.XMLHandler#parse(java.lang.Object, org.w3c.dom.Document)
	 */
	public Document parse(Object source, Document config) {
		Source s;
		if (source instanceof File) {
			s = new StreamSource((File)source);
		} else if (source instanceof InputStream) {
			s = new StreamSource((InputStream)source);
		} else if (source instanceof String) {
			s = new StreamSource(new ByteArrayInputStream(((String)source).getBytes()));
		} else {
			throw new RuntimeException("Cannot parse object of type " + source.getClass().getName());
		}
		DOMResult result = new DOMResult();
		try {
			transformer.transform(s, result);
		} catch (TransformerException e) {
			throw new RuntimeException("Cannot parse", e);
		}
		return (Document)result.getNode();
	}

	/* (non-Javadoc)
	 * @see org.avparade.xml.XMLHandler#write(java.lang.Object, org.w3c.dom.Document, org.w3c.dom.Document)
	 */
	public void write(Object dest, Document doc, Document config) {
		Source source = new DOMSource(doc);
		Result result;
		if (dest instanceof File) {
			result = new StreamResult((File)dest);
		} else if (dest instanceof OutputStream) {
			result = new StreamResult((OutputStream)dest);
		} else if (dest instanceof Writer) {
			result = new StreamResult((Writer)dest);
		} else {
			throw new RuntimeException("Cannot write to object of type " + dest.getClass().getName());
		}
		try {
			transformer.transform(source, result);
		} catch (TransformerException e) {
			throw new RuntimeException("Cannot write", e);
		}
	}

}
