/**
 * XMLHandlerBase.java
 *
 * Copyright (C) 2006 Paul R. Pierce
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see http://www.gnu.org/licenses/gpl.txt
 */
package org.avparade.xml;

import java.io.*;
import org.w3c.dom.*;

/**
 * org.avparade.xml.XMLHandlerBase
 *
 */
public abstract class XMLHandlerBase implements XMLHandler {

	
	/* (non-Javadoc)
	 * @see org.avparade.xml.XMLHandler#createDocument(java.lang.String, java.lang.String)
	 */
	public Document createDocument(String namespace, String documentElementName) {
		if (namespace != null && namespace.length() > 0) {
			return parse("<?xml version=\"1.0\" encoding=\"UTF-8\"?><" + documentElementName + " xmlns=\"" + namespace + "\"></" + documentElementName + ">", null);
		} else {
			return parse("<?xml version=\"1.0\" encoding=\"UTF-8\"?><" + documentElementName + "></" + documentElementName + ">", null);
		}
	}

	/* (non-Javadoc)
	 * @see org.avparade.xml.XMLHandler#toString(org.w3c.dom.Document, org.w3c.dom.Document)
	 */
	public String toString(Document doc, Document config) {
		StringWriter w = new StringWriter();
		write(w, doc, config);
		return w.toString();
	}

	/* (non-Javadoc)
	 * @see org.avparade.xml.XMLHandler#elementContents(org.w3c.dom.Element)
	 */
	public String elementContent(Element element) {
		StringBuffer sb = new StringBuffer();
		for (Node n = element.getFirstChild(); n != null; n = n.getNextSibling()) {
			if (n instanceof Text) {
				sb.append(((Text)n).getData());
			}
		}
		return sb.toString();
	}

	/* (non-Javadoc)
	 * @see org.avparade.xml.XMLHandler#firstElementChild(org.w3c.dom.Node, java.lang.String, java.lang.String)
	 */
	public Element firstElementChild(Node node, String namespace, String tag) {
		for (Node n = node.getFirstChild(); n != null; n = n.getNextSibling()) {
			if (n instanceof Element && (namespace == null || namespace.equals(((Element)n).getNamespaceURI())) && tag.equals(((Element)n).getTagName())) {
				return (Element)n;
			}
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.avparade.xml.XMLHandler#nextElementChild(org.w3c.dom.Node, java.lang.String, java.lang.String)
	 */
	public Element nextElementChild(Node node, String namespace, String tag) {
		for (Node n = node.getNextSibling(); n != null; n = n.getNextSibling()) {
			if (n instanceof Element && (namespace == null || namespace.equals(((Element)n).getNamespaceURI())) && tag.equals(((Element)n).getTagName())) {
				return (Element)n;
			}
		}
		return null;
	}

}
